import { Component, inject, OnInit } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import RedeemableAction from '../../shared/models/redeemable_action';
import { MatCardModule } from '@angular/material/card';
import { MAT_DIALOG_DATA, MatDialogModule, MatDialogRef } from '@angular/material/dialog';
import { MatButtonModule } from '@angular/material/button';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatSelectModule } from '@angular/material/select';
import { createItemExistsInArrayValidator } from '../../shared/validators/item-exists-in-array';
import { HermesClientService } from '../../hermes-client.service';

@Component({
  selector: 'action-item-edit',
  imports: [
    ReactiveFormsModule,
    MatButtonModule,
    MatCardModule,
    MatDialogModule,
    MatFormFieldModule,
    MatInputModule,
    MatSelectModule
  ],
  templateUrl: './action-item-edit.component.html',
  styleUrl: './action-item-edit.component.scss'
})
export class ActionItemEditComponent implements OnInit {
  readonly client = inject(HermesClientService);
  readonly dialogRef = inject(MatDialogRef<ActionItemEditComponent>);
  readonly data = inject<{ action: RedeemableAction, actions:RedeemableAction[] }>(MAT_DIALOG_DATA);
  action = this.data.action;
  actions = this.data.actions;
  readonly actionEntries: ({ [key: string]: any[] }) = {
    'SLEEP': [
      {
        key: 'sleep',
        type: 'number',
        label: 'Sleep (ms)',
        control: new FormControl(this.action.data['sleep'], [Validators.required, Validators.min(500)]),
      }
    ],
    'APPEND_TO_FILE': [
      {
        key: 'file_path',
        type: 'text',
        label: 'File Path',
        placeholder: '%userprofile%/Desktop/file.txt',
        control: new FormControl(this.action.data['file_path'], [Validators.required]),
      },
      {
        key: 'file_content',
        type: 'text',
        label: 'Content',
        placeholder: '%chatter% from %broadcaster%\'s chat says hi.',
        control: new FormControl(this.action.data['file_content'], [Validators.required]),
      },
    ],
    'WRITE_TO_FILE': [
      {
        key: 'file_path',
        type: 'text',
        label: 'File Path',
        placeholder: '%userprofile%/Desktop/file.txt',
        control: new FormControl(this.action.data['file_path'], [Validators.required]),
      },
      {
        key: 'file_content',
        type: 'text',
        label: 'Content',
        placeholder: '%chatter% from %broadcaster%\'s chat says hi.',
        control: new FormControl(this.action.data['file_content'], [Validators.required]),
      },
    ],
    'OBS_TRANSFORM': [
      {
        key: 'scene_name',
        type: 'text',
        label: 'OBS Scene Name',
        placeholder: 'Main Scene',
        control: new FormControl(this.action.data['scene_name'], [Validators.required]),
      },
      {
        key: 'scene_item_name',
        type: 'text',
        label: 'OBS Scene Item Name',
        placeholder: 'Item',
        control: new FormControl(this.action.data['scene_item_name'], [Validators.required]),
      },
      {
        key: 'position_x',
        type: 'text',
        label: 'Position X',
        placeholder: 'x + 50',
        control: new FormControl(this.action.data['position_x'], []),
      },
      {
        key: 'position_y',
        type: 'text',
        label: 'Position Y',
        placeholder: 'x - 166',
        control: new FormControl(this.action.data['position_y'], []),
      },
      {
        key: 'rotation',
        type: 'text',
        label: 'Rotation (in degrees)',
        placeholder: 'mod(x + 45, 360)',
        control: new FormControl(this.action.data['rotation'], []),
      },
    ],
    'AUDIO_FILE': [
      {
        key: 'file_path',
        type: 'text',
        label: 'Audio File Path',
        placeholder: '%userprofile%/Desktop/audio.mp3',
        control: new FormControl(this.action.data['file_path'], [Validators.required]),
      },
    ],
    'RANDOM_TTS_VOICE': [],
    'SPECIFIC_TTS_VOICE': [
      {
        key: 'tts_voice',
        type: 'text',
        label: 'TTS Voice Name',
        placeholder: 'Brian',
        control: new FormControl(this.action.data['tts_voice'], [Validators.required, Validators.minLength(2)]),
      },
    ],
    'TOGGLE_OBS_VISIBILITY': [
      {
        key: 'scene_name',
        type: 'text',
        label: 'OBS Scene Name',
        placeholder: 'Main Scene',
        control: new FormControl(this.action.data['scene_name'], [Validators.required]),
      },
      {
        key: 'scene_item_name',
        type: 'text',
        label: 'OBS Scene Item Name',
        placeholder: 'Item',
        control: new FormControl(this.action.data['scene_item_name'], [Validators.required]),
      },
    ],
    'SPECIFIC_OBS_VISIBILITY': [
      {
        key: 'scene_name',
        type: 'text',
        label: 'OBS Scene Name',
        placeholder: 'Main Scene',
        control: new FormControl(this.action.data['scene_name'], [Validators.required]),
      },
      {
        key: 'scene_item_name',
        type: 'text',
        label: 'OBS Scene Item Name',
        placeholder: 'Item',
        control: new FormControl(this.action.data['scene_item_name'], [Validators.required]),
      },
      {
        key: 'obs_visible',
        type: 'text-values',
        label: 'Visibility',
        values: ['visible', 'hidden'],
        control: new FormControl(this.action.data['scene_item_name'], [Validators.required]),
      },
    ],
    'SPECIFIC_OBS_INDEX': [
      {
        key: 'scene_name',
        type: 'text',
        label: 'OBS Scene Name',
        placeholder: 'Main Scene',
        control: new FormControl(this.action.data['scene_name'], [Validators.required]),
      },
      {
        key: 'scene_item_name',
        type: 'text',
        label: 'OBS Scene Item Name',
        placeholder: 'Item',
        control: new FormControl(this.action.data['scene_item_name'], [Validators.required]),
      },
      {
        key: 'obs_index',
        type: 'number',
        label: 'Visibility',
        control: new FormControl(this.action.data['scene_item_name'], [Validators.required, Validators.min(0)]),
      },
    ],
    'NIGHTBOT_PLAY': [],
    'VEADOTUBE_SET_STATE': [
      {
        key: 'state',
        type: 'text',
        label: 'Veadotube State name',
        placeholder: 'state #1',
        control: new FormControl(this.action.data['state'], [Validators.required]),
      },
    ],
  };
  readonly actionTypes = Object.keys(this.actionEntries);

  isNew: boolean = true;
  previousName: string = this.action.name;

  readonly formGroup = new FormGroup({
    name: new FormControl(this.action.name, [Validators.required]),
    type: new FormControl(this.action.type, [Validators.required]),
  });

  ngOnInit(): void {
    this.isNew = this.action.name.length <= 0;
    this.previousName = this.action.name;
    if (!this.isNew)
      this.formGroup.get('name')!.disable()
    else {
      this.formGroup.get('name')?.addValidators(createItemExistsInArrayValidator(this.actions, a => a.name));
    }
  }

  get exists(): boolean {
    return this.actions.some(a => a.name == this.action.name);
  }

  get formsValidity(): boolean {
    return this.formGroup.valid && this.action.type in this.actionEntries
      && this.actionEntries[this.action.type].every(f => f.control.valid);
  }

  get formsDirty(): boolean {
    return this.formGroup.dirty || this.action.type in this.actionEntries
      && this.actionEntries[this.action.type].some(f => f.control.dirty);
  }

  deleteAction(action: RedeemableAction): void {
    if (this.isNew)
      return;

    this.client.deleteRedeemableAction(action.name);
    this.dialogRef.close();
  }

  save(): void {
    if (this.formGroup.invalid) {
      return;
    }

    const fields = this.actionEntries[this.action.type];
    if (fields.some(f => f.control.invalid)) {
      return;
    }

    this.action.name = this.formGroup.get('name')!.value!;
    this.action.type = this.formGroup.get('type')!.value!;
    this.action.data = {}
    for (const entry of this.actionEntries[this.action.type]) {
      this.action.data[entry.key] = entry.control.value!.toString();
    }

    if (!(this.action.type in this.actionEntries)) {
      return;
    }

    this.dialogRef.close(this.action);
  }
}
